from palett.presets import PLANET
from palett.projector.projector import to_projector
from palett.projector.utils.bound_to_leap import bound_to_leap
from palett.projector.utils.preset_to_flat import preset_to_flat
from intype import is_numeric


def pigment(bound, preset=PLANET, effects=[]):
    vleap = bound_to_leap(bound)
    prime = preset_to_flat(preset)
    projector = to_projector(vleap, preset, effects)
    return lambda x: projector(x)(x) if is_numeric(x) else prime(x)



from .convert import \
    hex_hsl, hex_int, hex_rgb, \
    hsl_hex, hsl_rgb, \
    rgb_hex, rgb_hsl, rgb_int
from .fluo_entries import fluo_entries
from .fluo_matrix import fluo_matrix
from .fluo_vector import fluo_vector
from .preset import Preset



from aryth.enum_bound_keys import DIF, MIN
from ject.oneself import to_oneself
from palett.projector.utils.bound_to_leap import bound_to_leap
from palett.projector.utils.hsl_dyer import hsl_dyer
from palett.projector.utils.leverage import leverage
from palett.projector.utils.preset_to_leap import preset_to_leap


def to_projector(bound, preset, effects=None):
    if effects is None:
        effects = []
    if not bound or not preset:
        return to_oneself()
    bound = bound_to_leap(bound)
    leap = preset_to_leap(preset)
    if DIF not in bound or not bound[DIF]:
        dye = hsl_dyer(leap[MIN], *effects)
        return lambda _: dye
    return lambda x: projector(
        x,
        bound[MIN],
        leverage(leap[DIF], bound[DIF]),
        leap[MIN],
        effects
    )


def projector(x, m, lever, base, effects):
    r_h, r_s, r_l = lever
    m_h, m_s, m_l = base
    return hsl_dyer(
        (scale(x, m, r_h, m_h, 360),
         scale(x, m, r_s, m_s, 100),
         scale(x, m, r_l, m_l, 100),),
        *effects
    )


def scale(x, min_v, lever, base, ceil):
    return min((max(x, min_v) - min_v) * lever + base, ceil)



MAX = 'max'
MIN = 'min'
DIF = 'dif'


def bound_to_leap(bound):
    if MIN in bound:
        if DIF in bound and bound[DIF] is not None:
            pass
        elif MAX in bound and bound[MAX] is not None:
            bound[DIF] = bound[MAX] - bound[MIN]
    elif DIF in bound:
        if MAX in bound:
            bound[MIN] = bound[MAX] - bound[DIF]
        else:
            bound[MIN] = 0
    elif MAX in bound:
        bound[MIN] = 0
    else:
        bound[MIN] = bound[DIF] = 0
    return bound



from palett.convert.hex_int import hex_int

'''
@param {string} hex
@returns {number[]}
'''


def hex_rgb(hex_color):
    _int = hex_int(hex_color)
    return _int >> 16 & 0xFF, _int >> 8 & 0xFF, _int & 0xFF



from palett.convert.rgb_int import rgb_int

'''
@param {[number,number,number]} rgb
@returns {string}
'''


def rgb_hex(rgb):  # TODO: should convert to upper case
    return f'#{rgb_int(rgb):06x}'

    # return '#' + rgbToInt(rgb).toString(16).toUpperCase().padStart(6, '0')
